#
# spec file for package micropython
#
# Copyright (c) 2025 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%define pythons python3
%{?sle15_python_module_pythons}

Name:           micropython
Version:        1.25.0
Release:        3.1
Summary:        Implementation of Python 3 with very low memory footprint
License:        MIT
URL:            https://micropython.org/
Source0:        %{name}-%{version}.tar.xz
Source1:        prepare.sh
# PATCH-FIX-UPSTREAM
Patch0:         https://patch-diff.githubusercontent.com/raw/micropython/micropython/pull/17269.patch#/micropython-gcc15-string-initialization.patch
BuildRequires:  openssl
BuildRequires:  pkgconfig
BuildRequires:  python3
BuildRequires:  readline-devel
BuildRequires:  pkgconfig(libffi)

Recommends:     micropython-lib
ExcludeArch:    %{ix86} %{arm} ppc64 ppc64le

%package -n mpremote
Summary:        MicroPython remote control
BuildArch:      noarch
BuildRequires:  %{python_module hatch-requirements-txt}
BuildRequires:  %{python_module hatchling}
Requires:       python3-pyserial >= 3.3
BuildRequires:  %{python_module base >= 3.6}
BuildRequires:  %{python_module pip}
BuildRequires:  %{python_module wheel}
BuildRequires:  fdupes
BuildRequires:  python-rpm-macros

%package -n mpy-tools
Summary:        Tools for creating and handling precompiled .mpy files for MicroPython
Provides:       mpy-cross
Provides:       mpy-tool

%description
A lean and efficient Python implementation for microcontrollers and constrained systems

%description -n mpremote
This CLI tool provides an integrated set of utilities to remotely interact with
and automate a MicroPython device over a serial connection.

%description -n mpy-tools
MicroPython tools like the mpy-cross compiler for compiling.py files to .mpy files.
Also mpy-tool for inspecting .mpy files.

%prep
%autosetup -p1

sed -i -e "s:/usr/lib/micropython:%{_prefix}/lib/micropython:g" "ports/unix/main.c"

%define make_flags V=1 MICROPY_PY_BTREE=0 MICROPY_PY_USSL=0

%build
# micropython
export CFLAGS="%optflags -Wno-dangling-pointer"
%make_build -C mpy-cross
%make_build -C ports/unix STRIP=true
pushd tools/mpremote
# inject version info as there is no git checkout to get tags from
echo "VERSION = '%{version}'" > version.py
sed -i -e 's/source = "vcs"/path = "version.py"/' pyproject.toml
# remove useless shebang lines
sed -i -e 's_#!/usr/bin/env python3__' mpremote/{__main__,transport,transport_serial}.py
%pyproject_wheel
popd

%install
# micropython
install -d %{buildroot}%{_bindir}
install -t %{buildroot}%{_bindir} ports/unix/build-standard/micropython

# mpremote
pushd tools/mpremote
%pyproject_install
popd
%python_expand %fdupes %{buildroot}%{$python_sitelib}
# remove pycache to get rid of rpmlint "W: python-bytecode-inconsistent-mtime" warnings
%python_expand rm -rf %{buildroot}%{$python_sitelib}/mpremote/__pycache__

# mpy-tools
install -m755 -D -v mpy-cross/build/mpy-cross %{buildroot}%{_bindir}/mpy-cross
install -m755 -D -v tools/mpy-tool.py %{buildroot}%{_bindir}/mpy-tool
%python3_fix_shebang

%check
%ifnarch x86_64
# 2 tests fail: float_parse float_parse_doubleprec
# https://github.com/micropython/micropython/pull/6024
rm -f tests/float/float_parse.py
rm -f tests/float/float_parse_doubleprec.py
%endif
export MICROPY_CPYTHON3=python%python_version
make -C ports/unix PYTHON=%{_bindir}/python%python_version V=1 test

%files
%license LICENSE
%doc docs/unix/*
%{_bindir}/micropython

%files -n mpremote
%license tools/mpremote/LICENSE
%doc tools/mpremote/README.md
%{_prefix}/lib/python%{python_version}/site-packages/mpremote
%{_prefix}/lib/python%{python_version}/site-packages/mpremote-%{version}.dist-info
%{_bindir}/mpremote

%files -n mpy-tools
%doc mpy-cross/README.md
%license LICENSE
%{_bindir}/mpy-cross
%{_bindir}/mpy-tool

%changelog
* Thu Jul 10 2025 Atri Bhattacharya <badshah400@gmail.com>
- Add micropython-gcc15-string-initialization.patch: Refactor
  string literal as array initializer
  (gh#micropython/micropython#17269).
* Mon May 26 2025 Dominik Heidler <dheidler@suse.de>
- Add script to delete some 3rd party libraries from the src tar
  that are not needed to build the unix port in order to
  make the legal review easier
* Wed Apr 16 2025 Dominik Heidler <dheidler@suse.de>
- Update to 1.25.0
  * Add ROMFS support (see also "mpremote romfs" cmd with "query", "build", and "deploy" sub-commands)
    ROMFS defines a read-only, memory-mappable, extensible filesystem that can contain arbitrary resources,
  including precompiled mpy files, and allows executing bytecode directly from the filesystem.
  This makes importing significantly faster and use a lot less memory.
  * Inline assembler now supports 32-bit RISC-V assembly code via the newly implemented @micropython.asm_rv32 decorator.
  * Datagram TLS (DTLS) is now supported by the tls module and enabled on the alif, mimxrt, renesas-ra, rp2, stm32 and unix ports.
  * mpremote command-line tool now supports recursive remove via the new "rm -r"
  * mpremote now supports relative URLs in the package.json for installing from the local filesystem
  * "mpremote mount" has optimised readline support
  * full support for tuples and start/end arguments in str.startswith() and str.endswith() methods
  * vfs.mount() with no arguments now returns a list of mounted filesystems
  * marshal module has been added with dumps() and loads() functions
  * MicroPython native linker mpy_ld.py now includes support for linking in static libraries automatically
  * native modules now support 32-bit RISC-V code
  * force _FILE_OFFSET_BITS=64 to fix 32-bit file ABI
  * enable VfsRom on standard and coverage variants
  * use the bare metal mbedTLS config in the coverage buiid
  * add recursive mutex support
  * main: add coverage test for mounting ROMFS filesystem at startup
- Drop fix_re_sub_test_on_python3.13.patch as not needed anymore
* Mon Mar 17 2025 Dominik Heidler <dheidler@suse.de>
- Add fix_re_sub_test_on_python3.13.patch to fix re_sub test
* Tue Feb 18 2025 Dominik Heidler <dheidler@suse.de>
- Add subpackage mpy-tools which contains mpy-cross and mpy-tool
* Fri Feb  7 2025 Dominik Heidler <dheidler@suse.de>
- Fix build for 15.6
* Wed Feb  5 2025 Dominik Heidler <dheidler@suse.de>
- Build mpremote tool
* Sat Nov 30 2024 Jaime Marquínez Ferrándiz <jaime.marquinez.ferrandiz@fastmail.net>
- Update to 1.24.1:
  * https://github.com/micropython/micropython/releases/tag/v1.24.1
* Fri Nov 29 2024 Jaime Marquínez Ferrándiz <jaime.marquinez.ferrandiz@fastmail.net>
- Update to 1.24.0:
  * https://github.com/micropython/micropython/releases/tag/v1.24.0
* Wed Sep 18 2024 Yunus Acar <yunus.acar@suse.com>
- Update to 1.23.0:
  * https://github.com/micropython/micropython/releases/tag/v1.23.0
* Tue May 21 2024 Matej Cepl <mcepl@cepl.eu>
- Update to 1.22.2:
  This is a patch release containing the following commits:
  - py/compile: fix potential Py-stack overflow in try-finally
    with return
  - extmod/asyncio: support gather of tasks that finish early
  - extmod/modssl_mbedtls: fix cipher iteration in
    SSLContext.get_ciphers
  - extmod/btstack: reset pending_value_handle before calling
    write-done cb
  - extmod/btstack: reset pending_value_handle before calling
    read-done cb
  - esp32/mpnimbleport: release the GIL while doing NimBLE port
    deinit
  - esp32: increase NimBLE task stack size and overflow detection
    headroom
  - mimxrt/modmachine: fix deepsleep wakeup pin ifdef
  - renesas-ra/ra: fix SysTick clock source
  - renesas-ra/boards/ARDUINO_PORTENTA_C33: fix the RTC clock
    source
  - renesas-ra/ra/ra_i2c: fix 1 byte and 2 bytes read issue
  - rp2/rp2_dma: fix fetching 'write' buffers for writing not
    reading
  - rp2/machine_uart: fix event wait in uart.flush() and
    uart.read()
  - rp2: change machine.I2S and rp2.DMA to use shared DMA IRQ
    handlers
- Update to 1.22.1:
  - This is a patch release to fix a race condition and potential
    deadlock in the rp2 port's mp_thread_begin_atomic_section()
    function, when the second core is in use.
* Mon Jan 15 2024 Andreas Schwab <schwab@suse.de>
- Build with %%optflags
* Mon Jan  8 2024 Steve Kowalik <steven.kowalik@suse.com>
- Update to 1.22.0:
  * https://github.com/micropython/micropython/releases/tag/v1.22.0
  * objslice: validate that the argument to indices() is an integer
    (CVE-2023-7158, bsc#1218446)
* Fri Apr 28 2023 Steve Kowalik <steven.kowalik@suse.com>
- Update to 1.20.0:
  * https://github.com/micropython/micropython/releases/tag/v1.20.0
- Remove 32572439984e5640c6af46fbe7c27400c30112ce.patch, included.
- Disable warning, since it is treated as an error.
* Tue Mar 28 2023 Dirk Müller <dmueller@suse.com>
- add 32572439984e5640c6af46fbe7c27400c30112ce.patch
* Tue Oct 11 2022 Andreas Schwab <schwab@suse.de>
- Suppress stripping of binary
- Run tests everywhere with just the problematic ones suppressed
* Sat Jul  2 2022 Christophe Giboudeaux <christophe@krop.fr>
- Update to 1.19.1:
  * https://github.com/micropython/micropython/releases/tag/v1.19
* Wed Feb  2 2022 Jaime Marquínez Ferrándiz <jaime.marquinez.ferrandiz@fastmail.net>
- Update to 1.18: full changelog available at
  https://github.com/micropython/micropython/releases/tag/v1.18
* Sun Jan 16 2022 Dirk Müller <dmueller@suse.com>
- avoid spec-cleaner messing it up
* Fri Sep 24 2021 Matej Cepl <mcepl@suse.com>
- Update to 1.17:
  The release notes are REALLY long, so please go to
  https://github.com/micropython/micropython/releases/tag/v1.17
  to read them.
- Remove already upstreamed patch fix-uasyncio_heaplock-test.patch.
* Mon Aug  2 2021 Danilo Spinella <danilo.spinella@suse.com>
- Add fix-uasyncio_heaplock-test.patchto fix uasyncio_heaplock
  test (originally from gh#micropython/micropython#7449)
* Tue Jul 27 2021 Matej Cepl <mcepl@suse.com>
- Remove unnecessary dependency on python2
* Fri Jul 23 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Update to v1.16:
  * Full release notes: https://github.com/micropython/micropython/releases/tag/v1.16
  * New command-line tool called "mpremote"
  * Unix port updates:
  - modffi: use a union for passing/returning FFI values
  - main: increase stack limit on ARM architectures
  - modffi: fix conversion between Python integers and ffi types
  - fix build on arm64-darwin due to integer cast
* Fri May  7 2021 Mustafa Çalışkan <musfay@protonmail.com>
- Update to version 1.15:
  * Full release notes: https://github.com/micropython/micropython/releases/tag/v1.15
  * Unix port updates:
  - mpbtstackport_common: implement mp_bluetooth_hci_active
  - moduselect: don't allow both posix and non-posix configurations
  - improve command line argument processing
  - main: make static variable that's potentially clobbered by longjm
- Remove patch fix-aarch64-builds.patch
- Exclude ppc64 and ppc64le as build fails for those architectures
* Sat Apr 17 2021 Mustafa Çalışkan <musfay@protonmail.com>
- Add patch fix-aarch64-builds.patch
* Fri Feb  5 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Update to v1.14:
  * Full release notes: https://github.com/micropython/micropython/releases/tag/v1.14
  * Unix port updates:
  - Makefile: always enable -f*-sections regardless of DEBUG setting
  - implement BLE H4 HCI UART for btstack/nimble
  - implement mp_hal_time_ns using gettimeofday
  - variants: enable MICROPY_DEBUG_PARSE_RULE_NAME on coverage build
  - mpconfigport.h: enable MICROPY_PY_DELATTR_SETATTR
  - enable more warnings
  - support building C++ code
  - Makefile: move coverage.c and coveragecpp.cpp to coverage variant
  - handle pending events/scheduler in MICROPY_EVENT_POLL_HOOK
  - make mp_hal_delay_ms run MICROPY_EVENT_POLL_HOOK
  - modtime: fix time() precision on unix ports with non-double floats
* Fri Jan 29 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Drop _service file
- Exclude %%{arm} as lots of tests fail
* Wed Jan 27 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Cleanup package and enable tests
* Sun Sep  6 2020 Jaime Marquínez Ferrándiz <jaime.marquinez.ferrandiz@fastmail.net>
- Update to 1.13: full changelog available at https://github.com/micropython/micropython/releases/tag/v1.13
* Thu Dec 26 2019 Jaime Marquínez Ferrándiz <jaime.marquinez.ferrandiz@fastmail.net>
- Update to 1.12: full changelog available at https://github.com/micropython/micropython/releases/tag/v1.12
- Build mpy-cross
* Sun Jun 23 2019 Jaime Marquínez Ferrándiz <jaime.marquinez.ferrandiz@fastmail.net>
- Update to 1.11: full changelog at https://github.com/micropython/micropython/releases/tag/v1.11
* Sat Jan 26 2019 Jaime Marquínez Ferrándiz <jaime.marquinez.ferrandiz@fastmail.net>
- Update to 1.10: full changelog at https://github.com/micropython/micropython/releases/tag/v1.10
* Thu Dec  6 2018 Jaime Marquínez Ferrándiz <jaime.marquinez.ferrandiz@fastmail.net>
- Use _service
* Fri May 11 2018 jaime.marquinez.ferrandiz@fastmail.net
- Update to 1.9.4: full changelog at https://github.com/micropython/micropython/releases/tag/v1.9.4
* Fri Jan 12 2018 jaime.marquinez.ferrandiz@fastmail.net
- Update to 1.9.3: full changelog at https://github.com/micropython/micropython/releases/tag/v1.9.3
